Gamestate = require "libs.hump.gamestate"   -- Gamestate handling
Timer = require "libs.hump.timer"           -- Timed events
bump = require "libs.bump"                  -- Collision handler
require "conf"

screen = {
    W = love.graphics.getWidth(),
    H = love.graphics.getHeight()
}

local menu = {}
local game = {}

function switchState(state)
    Gamestate.switch(state)
end

function game:init()
player = {
    x = 40,
    y = screen.H/2-50,
    w = 15,
    h = 100,
    yvel = 0,
    speed = 5,
    direction = 1,
    wins = false,
    name = "player",
}
ai = {
    x = screen.W-player.w-40,
    y = screen.H/2-player.h/2,
    w = player.w,
    h = player.h,
    yvel = 0,
    CanMove = true,
    wins = false,
    name = "ai",
}
player_goal = {
    name = "player_goal",
    x = 0,
    y = 10,
    w = 10,
    h = screen.H-20
}
ai_goal = {
    name = "ai_goal",
    x = screen.W-10,
    y = 10,
    w = 10,
    h = screen.H-20
}
ball = {
    x = screen.W/2-5, -- 1/2 of radius
    y = screen.H/2,
    r = 10,
    xvel = 0,
    yvel = 0,
    speed = 300,
    maxvel = 10,
    direction = 1,
    xvel = 0,
    yvel = 0,
    collided = false,
    name = "ball"
}
score = {
    player = 0,
    ai = 0
}
top = {
    name="top",
    x = 0,
    y = 0,
    w = screen.W,
    h = 10
}
bottom = {
    name="bottom",
    x = 0,
    y = screen.H-10,
    w = screen.W,
    h = 10,
}

-- Bump init
world = bump.newWorld()
world:add(player, player.x, player.y, player.w, player.h) -- x,y, width, height
world:add(ai, ai.x, ai.y, ai.w, ai.h)
world:add(ball, ball.x, ball.y, ball.r, ball.r)
world:add(top, top.x, top.y, top.w, top.h)
world:add(bottom, bottom.x, bottom.y, bottom.w, bottom.h)
end

function game:update(dt)
-- Player controls
    if love.keyboard.isDown('w') or love.keyboard.isDown('up') then
        player.yvel = player.yvel + player.speed
        player.direction = -1
    elseif love.keyboard.isDown('s') or love.keyboard.isDown('down') then
        player.yvel = player.yvel + player.speed
        player.direction = 1
        else player.yvel = 0
    end
    -- Player direction (up/down)
    player.y = player.y + player.yvel * player.direction
    -- Limit paddle speed
    if player.yvel > player.speed then player.yvel = player.speed end
    -- Update player.x associated with bounding box
    world:move(player, player.x, player.y)
    -- Update the player associated bounding box in the world
    player.x, player.y, cols, len = world:move(player, player.x, player.y)

    -- Get angle of the ball relative to the center of the paddle
    function getAngle(paddle)
        -- sinQ=opposite/hypotenuse
        -- center of paddle, in x and y
        local centerX, centerY = paddle.x+paddle.w/2, paddle.y+paddle.h/2
         -- displacement between center of paddle and ball
        local hypotenuse = math.floor(math.sqrt((ball.x-centerX)^2+(ball.y-centerY)^2))
        -- height difference between ball & center of paddle
        local opposite = ball.y - centerY
         -- angle of theta, in degrees
        local angle = math.floor(math.deg(math.sin(opposite/hypotenuse)))
        return hypotenuse, opposite, angle
    end -- return values below
    pH, pO, pA = getAngle(player)
    aH, aO, aA = getAngle(ai)







    -- Ball controls
    ball.xvel = ball.xvel + ball.speed -- just so it initially goes into player paddle
    ball.x = ball.x - ball.xvel * dt
    -- Limit x/y velocity
    if ball.xvel > ball.speed then ball.xvel = ball.speed end
    if ball.yvel > ball.speed then ball.yvel = ball.speed end
    -- Move ball
    ball.x, ball.y, cols, len = world:move(ball, ball.x, ball.y, bounce) -- Bounce filter?
    if len > 0 then ball.collided = true else ball.collided = false end

    -- local actualX, actualY, cols, len = world:move(item, goalX, goalY, <filter>)








    -- AI controls
    if ai.CanMove == true then ai.yvel = ball.yvel end
    ai.x, ai.y, cols, len = world:move(ai, ai.x, ai.y) -- bump move ai

    -- Bump updates
    world:update(player, player.x, player.y)
    world:update(ai, ai.x, ai.y)
    world:update(ball, ball.x, ball.y)

    -- Scoring
    if ball.x < 0 then
        reset()
        score.ai = score.ai + 1
    elseif ball.x > screen.W then
        reset()
        score.player = score.player+1
    end
    if score.player == 5 or score.ai == 5 then
        if score.player == 5 then player.wins = true
        elseif score.ai == 5 then ai.wins = true
        end
        --endgame()
    end

    -- HUMP timer
   Timer.update(dt)
end

function endgame()
    ball.speed = 0
    Timer.after(3, function() os.exit() end)
end

function reset()
    ball.x = screen.W/2
end

function game:draw()
    love.graphics.setColor(255,255,255)

    -- Items
    love.graphics.rectangle("line", player.x, player.y, player.w, player.h)
    love.graphics.rectangle("fill", ai.x, ai.y, ai.w, ai.h)
    love.graphics.circle("fill", ball.x, ball.y, ball.r)

    -- Scores
    love.graphics.print(score.player, 20, 15)
    love.graphics.print(score.ai, screen.W-30, 15)

    -- Borders
    love.graphics.rectangle("fill", top.x, top.y, top.w, top.h)
    love.graphics.rectangle("fill", bottom.x, bottom.y, bottom.w, bottom.h)

    -- Feedback
    love.graphics.print(
        "player.y = " .. player.y .. "\n" ..
        "player.yvel = " .. player.yvel * player.direction .. "\n" ..
        "ai.y = " .. ai.y .. "\n" ..
        "ai.yvel = " .. ai.yvel .. "\n" ..
        "ball.x = " .. math.floor(ball.x) .. "\n" ..
        "ball.y = " .. ball.y .. "\n" ..
        "ball.yvel = " .. ball.yvel .. "\n" ..
        "ball.xvel = " .. ball.xvel .. "\n" ..
        "ball.collision = " .. tostring(ball.collided) .. "\n\n" ..
        -- Angle settings
        "player hyp. = " .. pH .. "\n" ..
        "player opp. = " .. pO .. "\n" ..
        "player ang. = " .. pA .. "\n\n" ..

        "ai hyp. = " .. aH .. "\n" ..
        "ai opp. = " .. aO .. "\n" ..
        "ai ang. = " .. aA .. "\n",

        80, 20)

    love.graphics.setColor(255,0,255)
    love.graphics.rectangle("fill", player_goal.x, player_goal.y, player_goal.w, player_goal.h)
    love.graphics.rectangle("fill", ai_goal.x, ai_goal.y, ai_goal.w, ai_goal.h)
    love.graphics.rectangle("fill", ball.x, ball.y, ball.r, ball.r) -- Bounding box of circle


    -- Center of paddles
    love.graphics.rectangle("fill", player.x+player.w/2, player.y+player.h/2, 3, 3)
    love.graphics.rectangle("fill", ai.x+ai.w/2, ai.y+ai.h/2, 3, 3)

    -- Scoring alert
    if player.wins == true then love.graphics.printf("Player wins!",screen.W/2,screen.H/2-40) end
    if ai.wins == true then love.graphics.print("AI wins!", screen.W/2,screen.H/2-40) end
end

function love.load()
    Gamestate.registerEvents()
    switchState(game)
end